/* ------------------ Programme principal ------------------*/
#define ERROR_CHECK_LEVEL 2

#include <stdio.h>
#include <Pilot.h>
#include <Common.h>
#include <MemoryMgr.h>
#include <FeatureMgr.h>
#include <KeyMgr.h>
#include <BuildRules.h>
#include "callback.h"
#include "rc.h"

#include "global.c"
#include "6502.c"

// ================ gestion des evenements ================

static Boolean MainEventHandler(EventPtr e)
{
	Boolean handled = true;
	Char texte[40];
	Char val[9];

	CALLBACK_PROLOGUE
	
	switch (e->eType) {

	case frmOpenEvent :
		FrmDrawForm(FrmGetActiveForm());

		handled = true;
		break;

	case keyDownEvent:
		Touche = e->data.keyDown.chr;	/* Touche enfoncee */
/*
		StrCopy (texte, "Touche xx");
		StrIToH (val, Touche);
		texte[7] = val[6];
		texte[8] = val[7];
		ErrDisplay (texte);
*/
		if ((Touche >= 0x61) && (Touche <= 0x7A)) Touche = Touche - 0x20;			/* Lettres majuscules */
		if (Touche == 0x0A) Touche = 0x0D;			/* Touche RETURN */
		if (Touche == 0x08) Touche = 0x7F;			/* Touche BACKSPACE */
		putROM (0xF4AF, Touche);	/* Force la touche */
		putROM (0xF51E, 0x08);	/* Touche enfoncee = 0x08, sinon 0x00 */

		handled = true;
		break;

	default :
		handled = false;
	}


	/* ---- Emulation ---- */
	/* Execute quelques instructions */
	exec6502 ();


	CALLBACK_EPILOGUE
	
	return handled;
}

static Boolean AppHandleEvent(EventPtr e)
{
	Word formID;
	FormPtr frm;

	Boolean handled = false;

	if (e->eType == frmLoadEvent) {
		formID = e->data.frmLoad.formID;
		frm = FrmInitForm(formID);
		FrmSetActiveForm(frm);
		switch (formID) {
			case idFormMain :
				FrmSetEventHandler(frm, MainEventHandler);
				break;
		}
		handled = true;
	}
	return handled;
}

static void ProcessEvent(EventPtr e)
{
	Word error;

	if (SysHandleEvent(e)) return;
	if (MenuHandleEvent(NULL, e, &error)) return;
	if (AppHandleEvent(e)) return;
	FrmDispatchEvent(e);
}

static void EventLoop(void)
{
	EventType event;

	do {
		EvtGetEvent(&event, 0);		/* Recupere un evenement sans attendre */
		ProcessEvent(&event);
	} while (event.eType != appStopEvent);
}

static Boolean StartApplication(void)
{
	FrmGotoForm(idFormMain);

	/* Init de l'emulateur */
	initORIC();

	/* must be called first to initialise all 6502 engines arrays */
	init6502();

	/* sets all of the 6502 registers. The program counter is set from
	 * locations $FFFC and $FFFD masked with the above addrmask */
	reset6502();

	return true;
}

static void StopApplication(void)
{
	/* Fin de l'emulateur */
	exitORIC();
}

DWord PilotMain(Word cmd, Ptr cmdPBP, Word launchFlags)
{
	if (cmd == sysAppLaunchCmdNormalLaunch) {
		if (StartApplication()) {
			EventLoop();
			StopApplication();
		}
	}
	return 0;
}

